@@ -5,12 +5,16 @@
 #'
 #' @param filepath, string. Full path to the output curv file. If it ends with ".gz", the file is written in gzipped format. Note that this is not common, and that other software may not handle this transparently.
 #'
-#' @param data vector of floats. The brain morphometry data to write, one value per vertex.
+#' @param data vector of doubles. The brain morphometry data to write, one value per vertex.
 #'
 #' @family morphometry functions
 #'
 #' @export
 write.fs.curv <- function(filepath, data) {
+    if(! is.double(data)) {
+      warning("Parameter 'data' is not of type double, trying to coerce.");
+      data = as.double(data);
+    }
     MAGIC_FILE_TYPE_NUMBER = 16777215;
     num_verts = length(data);
     num_faces = length(data);   # Has no meaning.
@@ -80,7 +84,7 @@ write.fs.morph <- function(filepath, data, format='auto', ...) {
     if(format == "mgh" || format == "mgz" ) {
         write.fs.mgh(filepath, data, ...);
     } else if (format == "gii") {
-        stop("Writing files in GIFTI format is not supported.");
+        write.fs.morph.gii(filepath, data);
     } else if (format == "curv") {
         write.fs.curv(filepath, data);
     }
@@ -88,6 +92,27 @@ write.fs.morph <- function(filepath, data, format='auto', ...) {
 }
 
 
+#' @title Write morphometry data in GIFTI format.
+#'
+#' @description The data will be written with intent 'NIFTI_INTENT_SHAPE' and as datatype 'NIFTI_TYPE_FLOAT32'.
+#'
+#' @param filepath string, the full path of the output GIFTI file.
+#'
+#' @param data numerical vector, the data to write. Will be coerced to double.
+#'
+#' @return format, string. The format that was used to write the data: "gii".
+#'
+#' @family morphometry functions
+#' @family gifti writers
+#'
+#' @export
+write.fs.morph.gii <- function(filepath, data) {
+  data = as.double(data);
+  gifti_writer(filepath, list(data), intent='NIFTI_INTENT_SHAPE', datatype='NIFTI_TYPE_FLOAT32');
+  return(invisible('gii'));
+}
+
+
 #' @title Determine morphometry file format from filename
 #'
 #' @description Given a morphometry file name, derive the proper file format, based on the end of the string. Case is ignored, i.e., cast to lowercase before checks. If the filepath ends with "mgh", returns format "mgh". For suffix "mgz", returns "mgz" format. For all others, returns "curv" format.
@@ -134,7 +159,7 @@ fs.get.morph.file.format.from.filename <- function(filepath) {
 #'
 #' @description Given a morphometry file format, derive the proper file extension.
 #'
-#' @param format, string. One of c("mgh", "mgz", "curv").
+#' @param format, string. One of c("mgh", "mgz", "curv", "gii").
 #'
 #' @return file ext, string. The standard file extension for the format. (May be an empty string for some formats.)
 #'
@@ -148,6 +173,8 @@ fs.get.morph.file.ext.for.format <- function(format) {
         return(".mgz");
     } else if(format == "curv") {
         return("");
+    } else if(format == "gii") {
+      return(".gii");
     } else {
         stop(sprintf("Unsupported morphometry file format: '%s'.", format));
     }
