@@ -39,8 +39,8 @@ test_that("One can read, write and re-read triangular surface data", {
 
   skip_if(tests_running_on_cran_under_macos(), message = "Skipping on CRAN under MacOS, required test data cannot be downloaded.");
 
-  freesurferformats::download_optional_data();
-  subjects_dir = freesurferformats::get_optional_data_filepath("subjects_dir");
+  freesurferformats::download_opt_data();
+  subjects_dir = freesurferformats::get_opt_data_filepath("subjects_dir");
   surface_file = file.path(subjects_dir, "subject1", "surf", "lh.white");
 
   skip_if_not(file.exists(surface_file), message="Test data missing.");
@@ -82,7 +82,7 @@ test_that("Surface files in VTK format can be read and written", {
 })
 
 
-test_that("One can export surface meshes in OFF, OBJ and PLY formats", {
+test_that("One can export surface meshes in OFF, OBJ, PLY2 and PLY formats", {
 
   surface_file = system.file("extdata", "lh.tinysurface", package = "freesurferformats", mustWork = TRUE);
   mesh = read.fs.surface(surface_file);
@@ -97,6 +97,9 @@ test_that("One can export surface meshes in OFF, OBJ and PLY formats", {
   # OFF, the Object File Format
   write.fs.surface.off(tempfile(fileext=".off"), mesh$vertices, mesh$faces);
 
+  # PLY2 format, very similar to OFF.
+  write.fs.surface.ply2(tempfile(fileext=".ply2"), mesh$vertices, mesh$faces);
+
   # Wavefront OBJ format
   write.fs.surface.obj(tempfile(fileext=".obj"), mesh$vertices, mesh$faces);
 
